(function () {
'use strict';

function registerPluginCommands(commands){
    var lowerCaseCommands = {};
    Object.keys(commands).forEach(function (name){
        lowerCaseCommands[name.toLowerCase()] = commands[name];
    });

    var Game_Interpreter_prototype_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function(cmd, args){
        var command = lowerCaseCommands[cmd.toLowerCase()];
        if(command){
            command.apply(this, args);
        }

        return Game_Interpreter_prototype_pluginCommand.call(this, cmd, args);
    };
}



function wrapStatic(klass, method, fn){
    var oldMethod = klass[method];
    var newMethod = fn(oldMethod);

    klass[method] = newMethod;
}

/*:
 * @author liply
 * @help
 * 以下のプラグインコマンドを導入します。
 *
 * sql
 * sqlを発行します。
 *
 * cursor fetch [from] [to] [column] [offset]
 * カーソルからデータを取り出します。
 * columnで指定したデータが、[from-to)の変数に書き込まれます。
 * オフセットを指定したい場合はoffsetを利用します。
 *
 * cursor count [id]
 * 発行したsqlが持つデータの数をidで指定した変数に格納します。
 *
 */

var C = [];

wrapStatic(DataManager, 'loadDatabase', function (old){ return function(){
    if(window.__liply_DatabaseDevelopment){
        window.__liply_DatabaseDevelopment.then(function (){
            DataManager.loadDataFile('$dataDatabase', 'Database.json');
        });
        window.__liply_DatabaseDevelopment = null;
    }else{
        DataManager.loadDataFile('$dataDatabase', 'Database.json');
    }
    old.call(this);
}; });

registerPluginCommands({
    sql: function sql(){
        var params = [], len = arguments.length;
        while ( len-- ) params[ len ] = arguments[ len ];

        var sql = params.join(' ');
        C = $dataDatabase[sql];
    },
    cursor: function cursor(command){
        var params = [], len = arguments.length - 1;
        while ( len-- > 0 ) params[ len ] = arguments[ len + 1 ];

        switch(command.toLowerCase()){
            case 'fetch': {
                var from = +params[0];
                var to = +params[1];
                var column = params[2];
                var offset = +params[3] || 0;
                for(var n = 0; n < to - from; n++){
                    $gameVariables.setValue(n+from, C[n+offset]? C[n+offset][column]: null);
                }
            }
            break;

            case 'count': {
                var id = +params[0];
                $gameVariables.setValue(id, C.length);
            }
        }
    }
});

}());
